/**
 * RO_Figure.cpp
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * An RO_Figure is a character in the game Ragnarok Online that can be drawn
 * with various skill/timer effects; it can also represent monsters too!
 *
 * Original Source: N/A
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#include "include.h"

/**
 * Construct a new RO_Figure by allocating arrays for its wireframe mesh.
 */
RO_Figure::RO_Figure() : x_pos(0.0), y_pos(0.0), z_pos(0.0), dex(150), 
        castTimePer(0.0), castActive(false), delayActive(false), 
        auraActive(false), nameActive(false), mat(0)
{
    triangles = new vector<Tri *>[13];
    points = new vector<Point3 *>[13];
    quad = gluNewQuadric();
}

/**
 * Destruct the allocated wireframe arrays and the quadric object.
 */
RO_Figure::~RO_Figure()
{
    // FOR every body part
    for(int i = 0; i < 13; i++)
    {
        // FOR all triangles for that part
        for(int j = 0; j < triangles[i].size(); j++)
        {
            delete triangles[i][j];
        }
        // FOR all points for that part
        for(int k = 0; k < points[i].size(); k++)
        {
            delete points[i][k];
        }
    }
    delete[] triangles;
    delete[] points;
    gluDeleteQuadric(quad);
}

/**
 * Increase this figure's DEX stat in order to decrease cast time.
 */
void RO_Figure::incDex() 
{
    // Maximum DEX is 150 (0% cast time)
    if(dex < 150) 
    {
        dex += 5;
        castTimePer = 1-(float)dex/15*10/100;
        printf("DEX increased to %d.\n", dex);
    }
}

/**
 * Decrease this figure's DEX stat in order to increase cast tiem.
 */
void RO_Figure::decDex()
{
    // Minimum DEX is 0 (100% cast time)
    if(dex > 0) 
    {
        dex -= 5;
        castTimePer = 1-(float)dex/15*10/100;
        printf("DEX decreased to %d.\n", dex);
    }
} 


/**
 * Process each line of input and save vertices and faces appropriately.
 * fig:  Related figure to save vertices and faces for.
 * str:  Input line to process.
 * points:  Array of body part points.
 * triangles:  Array of body part faces.
 * partID:  The body part we are saving vertices and faces for.
 */
void readLine(RO_Figure * fig, char * str, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID)
{
    int indx = 0, vi;
    float x, y, z;
    float r, g, b;
    int mat;

    if (str[0]=='#')
    {
        return;
    }
    // Read a vertex or face
    if (str[0]=='V' && !strncmp(str,"Vertex ",7))
    {
        Point3* p;
        assert(sscanf(str,"Vertex %d %g %g %g",&vi,&x,&y,&z) == 4);
        p = new Point3(x, y, z);

        // Store the vertex
        points[partID].push_back(p);
    }
    else if (str[0]=='F' && !strncmp(str,"Face ",5))
    {
        Tri* t;
        t = new Tri();
        char * s = str+4;
        int fi = -1;
        for(int t_i = 0; ; t_i++)
        {
            while (*s && isspace(*s))
                s++;
            // If we reach the end of the line break out of the loop
            if (!*s)
                break;

            // Save the position of the current character
            char * beg = s;

            // Advance to next space
            while (*s && isdigit(*s))
                s++;

            // Covert the character to an integer
            int j = atoi(beg);

            // The first number we encounter will be the face index, don't
            //store it
            if (fi<0) { fi=j; continue; }

            // Otherwise process the digit we've grabbed in j as a vertex index
            // The first number will be the face id the following are vertex ids
            if (t_i == 1) t->v1 = j;
            else if (t_i == 2) t->v2 = j;
            else if (t_i == 3) t->v3 = j;

            // If there is more data to process break out
            if (*s =='{') break;
        }
        // Possibly process colors if the mesh has colors
        if (*s && *s =='{')
        {
            char *s1 = s + 1;

            // If we're reading off a color
            if (!strncmp(s1,"rgb=",4))
            {
                // Grab the values of the string
                assert(sscanf(s1,"rgb=(%g %g %g) matid=%d",&r,&g,&b,&mat) == 4);
                t->color.x = r; t->color.y = g; t->color.z = b;
            }
        }
        // Calculate the normal for the triangle
        calcNormal(t, points, partID);

        // Store the triangle read in
        triangles[partID].push_back(t);
    }
}

/**
 * Process the input stream from an input stream.
 * fig:  Related figure to read vertices and faces for.
 * is:  the stream to read from.
 * points:  Array of body part points.
 * triangles:  Array of body part faces.
 * partID:  The body part we are reading vertices and faces for.
 */
void readStream(RO_Figure * fig, istream& is, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID)
{
    char str[256];
    for (;is;)
    {
      is >> ws;
      is.get(str,sizeof(str));
      if (!is) break;
      is.ignore(9999,'\n');
      readLine(fig, str, points, triangles, partID);
    }
}

/**
 * Open a file for reading coordinates.
 * fig:  Related figure to read vertices and faces for.
 * filename:  the file to open.
 * points:  Array of body part points.
 * triangles:  Array of body part faces.
 * partID:  The body part we are reading vertices and faces for.
 */
void readFile(RO_Figure * fig, char * filename, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID)
{
    printf("Reading coordinates from %s\n", filename);

    ifstream in_f(filename);
    // IF the file could not be found
    if (!in_f)
    {
        printf("Could not open file %s\n", filename);
    }
    // ELSE read the file
    else
    {
        readStream(fig, in_f, points, triangles, partID);
    }
    // Calculate the vertex normals for all body parts (smooth shading)
    calcAllVertexNormals(points, triangles, partID);
}

/**
 * Calculate the normals for every vertex.
 * points:  Array of body part points.
 * triangles:  Array of body part faces.
 * partID:  The body part we are calculating vertex normals for.
 */
void calcAllVertexNormals(vector<Point3 *> * points, vector<Tri *> * triangles,
    int partID)
{
    // For each face
    for(int i = 0; i < triangles[partID].size(); i++)
    {
        // Add that face's normal to each vertex's normal
        points[partID][triangles[partID][i]->v1]->normal.x 
            += triangles[partID][i]->normal.x;
        points[partID][triangles[partID][i]->v1]->normal.y 
            += triangles[partID][i]->normal.y;
        points[partID][triangles[partID][i]->v1]->normal.z 
            += triangles[partID][i]->normal.z;

        points[partID][triangles[partID][i]->v2]->normal.x 
            += triangles[partID][i]->normal.x;
        points[partID][triangles[partID][i]->v2]->normal.y 
            += triangles[partID][i]->normal.y;
        points[partID][triangles[partID][i]->v2]->normal.z 
            += triangles[partID][i]->normal.z;

        points[partID][triangles[partID][i]->v3]->normal.x 
            += triangles[partID][i]->normal.x;
        points[partID][triangles[partID][i]->v3]->normal.y 
            += triangles[partID][i]->normal.y;
        points[partID][triangles[partID][i]->v3]->normal.z 
            += triangles[partID][i]->normal.z;
    }

    // For each vertex
    for(int j = 0; j < points[partID].size(); j++)
    {
        // Normalize that normal
        float length = sqrt(
              points[partID][j]->normal.x*points[partID][j]->normal.x
            + points[partID][j]->normal.y*points[partID][j]->normal.y
            + points[partID][j]->normal.z*points[partID][j]->normal.z);
        points[partID][j]->normal.x /= length;
        points[partID][j]->normal.y /= length;
        points[partID][j]->normal.z /= length;
    }
}

/**
 * Drawing routine to draw triangles as wireframe.
 * t:  Triangle to draw.
 * partID:  The body part related to this triangle.
 */
void RO_Figure::drawTriangle(Tri* t, int partID)
{
    glBegin(GL_POLYGON);
    glVertex3f(points[partID][t->v1]->x,
      points[partID][t->v1]->y,
      points[partID][t->v1]->z);

    glVertex3f(points[partID][t->v2]->x,
      points[partID][t->v2]->y,
      points[partID][t->v2]->z);

    glVertex3f(points[partID][t->v3]->x,
      points[partID][t->v3]->y,
      points[partID][t->v3]->z);
    glEnd();
}

/**
 * Draw a body part for this figure.
 * partID:  The body part to draw.
 */
void RO_Figure::drawPart(int partID)
{
    // Draw the wireframe mesh
    for(int j = 0; j < triangles[partID].size(); j++)
    {
        glColor3f(triangles[partID][j]->color.x, 
            triangles[partID][j]->color.y, triangles[partID][j]->color.z);
        drawTriangle(triangles[partID][j], partID);
    }
}