/**
 * RO_Figure.h
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * An RO_Figure is a character in the game Ragnarok Online that can be drawn
 * with various skill/timer effects; it can also represent monsters too!
 *
 * Original Source: N/A
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#ifndef RO_FIGURE_H
#define RO_FIGURE_H

#include "include.h"
using namespace std;

#define RO_UPPER_HEADGEAR 0
#define RO_MIDDLE_HEADGEAR 1
#define RO_LOWER_HEADGEAR 2
#define RO_TORSO 3
#define RO_HEAD 4
#define RO_RIGHT_ARM 5
#define RO_LEFT_ARM 6
#define RO_RIGHT_HAND 7
#define RO_LEFT_HAND 8
#define RO_RIGHT_FOOT 9
#define RO_LEFT_FOOT 10
#define RO_RIGHT_LEG 11
#define RO_LEFT_LEG 12

class RO_Figure
{
public:
    vector<Tri *> * triangles;  // Triangle mesh for this figure
    vector<Point3 *> * points;  // Vertices for this figure

    int mat;                    // Color value from file
    int skill;                  // Skill being casted
    // Indicates if a skill delay is currently being incurred
    bool delayActive;
    DWORD initDelayTime;        // Initial delay time

    RO_Figure();
    ~RO_Figure();
    virtual void draw(int theta, float omega) = 0;
    void drawTriangle(Tri* t, int partID);
    void drawPart(int partID);
    void incDex();
    void decDex();
    
protected:
    float x_pos, y_pos, z_pos;  // Position of the figure in the world

    int dex;                    // DEX stat 
   
    // Percent cast time (for timer reduction bonuses from stats like DEX)
    float castTimePer;
    // Fraction of time a skill cast aura has elapsed
    float auraFrac;

    // Indicates if a skill is currently being casted
    bool castActive;
    // Indicates if a skill aura is currently animating
    bool auraActive;
    // Indicates if a skill name is currently drawn
    bool nameActive;

    char * skillName;          // Skill name to display in the black box
    DWORD initCastTime;        // Initial cast time
    GLUquadricObj * quad;      // Quadric object for drawing

    virtual void draw_upper_headgear() = 0;
    virtual void draw_middle_headgear() = 0;
    virtual void draw_lower_headgear() = 0;
    virtual void draw_torso() = 0;
    virtual void draw_head() = 0;
    virtual void draw_right_arm() = 0;
    virtual void draw_left_arm() = 0;
    virtual void draw_right_hand() = 0;
    virtual void draw_left_hand() = 0;
    virtual void draw_right_foot() = 0;
    virtual void draw_left_foot() = 0;
    virtual void draw_right_leg() = 0;
    virtual void draw_left_leg() = 0;
    virtual void draw_skill_timer(float angle, float percent) = 0;
};

void readLine(RO_Figure * fig, char * str, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID);
void readStream(RO_Figure * fig, istream& is, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID);
void readFile(RO_Figure * fig, char * filename, vector<Point3 *> * points, 
    vector<Tri *> * triangles, int partID);
void calcAllVertexNormals(vector<Point3 *> * points, vector<Tri *> * triangles,
    int partID);

#endif
