/**
 * RO_Monk.h
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * A monk is a character class in the MMORPG Ragnarok Online.  It can be drawn
 * in the world as a human figure with assorted headgear, spirit spheres, and
 * fury aura.  Certain skill animations are also activated when the keyboard
 * is pressed (the keyboard function is in main_release.cpp).
 *
 * Original Source: N/A
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#ifndef RO_MONK_H
#define RO_MONK_H

#include "include.h"

static const int NUM_SKILLS = 3; // Number of available skills to cast

/** Skill identifiers */
static const int SPHERE = 0;
static const int FURY   = 1;
static const int GFIST  = 2;

/** Base times for skill casting timers in msec */
static const int TIME_SPHERE = 1000;
static const int TIME_FURY   = 0;
static const int TIME_GFIST  = 4000;

/** Times for skill cast animations in msec */
static const int TIME_SPHERE_ANIM = 1000;
static const int TIME_FURY_ANIM = 2000;
static const int TIME_GFIST_ANIM = 6000;

/** Base times for after-skill delay timers in msec */
static const int DELAY_SPHERE = 0;
static const int DELAY_FURY   = 1000;
static const int DELAY_GFIST  = 2000;

/** Skill names for display */
static char * SKILL_NAMES[NUM_SKILLS] = {"Summon Spirit Sphere !!", 
    "Fury !!", "Guillotine Fist !!"};

/** Abstraction of times for easier programming use */
static const unsigned int SKILL_TIMES[NUM_SKILLS] = 
    {TIME_SPHERE, TIME_FURY, TIME_GFIST};
static const unsigned int SKILL_ANIMS[NUM_SKILLS] = 
    {TIME_SPHERE_ANIM, TIME_FURY_ANIM, TIME_GFIST_ANIM};
static const unsigned int SKILL_DELAYS[NUM_SKILLS] = 
    {DELAY_SPHERE, DELAY_FURY, DELAY_GFIST};

/** Time (in msec) that the black skill name box will stay drawn */
static const int TIME_SKILL_NAME = 6000;

class RO_Monk : public RO_Figure
{
    void draw_upper_headgear();
    void draw_middle_headgear();
    void draw_lower_headgear();
    void draw_torso();
    void draw_head();
    void draw_right_arm();
    void draw_left_arm();
    void draw_right_hand();
    void draw_left_hand();
    void draw_right_foot();
    void draw_left_foot();
    void draw_right_leg();
    void draw_left_leg();
    void draw_spheres(int theta);
    void draw_fury_state(int theta);

    void draw_sphere_cast_aura();
    void draw_fury_cast_aura();
    void draw_gfist_cast_aura();
    void draw_gfist_chars(float angle);
    void draw_gfist_blast();

    void draw_skill_name(float angle);
    void draw_skill_timer(float angle, float percent);

    int numSpheres;             // Number of active spheres on this monk
    bool furyState;             // Indicates if the monk is in fury state
    Texture * gfist;            // Guillotine fist characters

    /**
     * A GF_Wave is a spoke of the blast aura enacted by the completion of a
     * Guillotine Fist.  Think of it as one stream of a fireworks display.
     */
    class GF_Wave
    {
    public:
        float time;     // Time it appears (as percent of delay)
        float length;   // How long the wave is
        float angle;    // Angle at which this wave expands from the monk

        GF_Wave()
        {
            time = (float)rand()/RAND_MAX * 0.5;
            length = (float)rand()/RAND_MAX * 4.0;
            angle = (float)rand()/RAND_MAX * 360;
        }
    };

    GF_Wave * wave;             // Wave explosion

public:
    RO_Monk();
    ~RO_Monk() {delete gfist; delete[] wave;}
    void draw(int theta, float omega);
    void summonSpiritSphere();
    void fury();
    void guillotineFist();
};

#endif