/**
 * RO_Poring.cpp
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * A poring is a monster in the MMORPG Ragnarok Online.  It explodes into
 * little globs when killed.
 *
 * Original Source: N/A
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#include "include.h"

/**
 * Construct a poring with a monk as an attacker.
 * amonk:  The monk attacking this poring.
 */
RO_Poring::RO_Poring(RO_Monk * amonk)
{
    // Give the poring an initial position
    x_pos = -3.0; 
    y_pos = 0.0; 
    z_pos = 3.0;
    monk = amonk;

    // Read in the poring mesh
    readFile(this, "poring.m", points, triangles, RO_TORSO);
}

/**
 * Draw this poring.
 * theta:  Rotation angle for animations.
 * omega:  Angle between <0, 0, 1> and the camera projected onto y = 0.  
 * This is for drawing any skill words such that they're facing the direction
 * of the camera.
 */
void RO_Poring::draw(int theta, float omega)
{
    m.init_materials();

    // IF the monk casted a Guillotine Fist, draw the poring death
    if(monk->delayActive && monk->skill == GFIST)
    {
        glPushMatrix();
            draw_death();
        glPopMatrix();
    }
    // ELSE draw a stationary poring
    else
    {
        glDisable(GL_LIGHTING);
        glPushMatrix();
            // Place the poring in the world
            glTranslatef(x_pos, y_pos, z_pos);
            draw_torso();
        glPopMatrix();
        glEnable(GL_LIGHTING);
    }
}

/** Unused functions for the poring model. */
void RO_Poring::draw_upper_headgear() {}
void RO_Poring::draw_middle_headgear() {}
void RO_Poring::draw_lower_headgear() {}
void RO_Poring::draw_head() {}
void RO_Poring::draw_right_arm() {}
void RO_Poring::draw_left_arm() {}
void RO_Poring::draw_right_hand() {}
void RO_Poring::draw_left_hand() {}
void RO_Poring::draw_right_foot() {}
void RO_Poring::draw_left_foot() {}
void RO_Poring::draw_right_leg() {}
void RO_Poring::draw_left_leg() {}
void RO_Poring::draw_skill_timer(float angle, float percent) {}


/**
 * Draw the poring body.
 */
void RO_Poring::draw_torso()
{
    glTranslatef(0.0, 1.0, 0.0);
    glScalef(0.5, 0.5, 0.5);
    drawPart(RO_TORSO);
}

/**
 * Draw the poring death animation.
 */
void RO_Poring::draw_death()
{
    // Percentage of delay time elapsed since the monk finished casting a
    // Guillotine Fist
    float blobTime = (GetTickCount() - monk->initDelayTime)/(float)DELAY_GFIST;

    float newX, newY; // Coordinates for flying poring bits

    // FOR five poring pieces
    for(int i = 0; i < 5; i++)
    {
        glPushMatrix();
            // Translate to previous poring position
            glTranslatef(x_pos, y_pos, z_pos);
            glPushMatrix();
                m.materials(m.pink);
                // Rotate around
                glRotatef(360/5*i, 0, 1, 0);
                // Translate via pseudo-projectile explosion
                newX = 5.0*blobTime;
                newY = (-((blobTime)*(blobTime)) + .5);
                glTranslatef(newX, newY, 0.0);
                // Draw a poring piece
                glutSolidSphere(0.5, 50, 50);
            glPopMatrix();
        glPopMatrix();
    }
}