/**
 * Texture.cpp
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * A Texture is a bitmap (size power of 2) used to create landscapes or
 * other situations that require the use of texture mapping.
 *
 * Original Source: ZJW lab 11 code.
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#include "include.h"

/** 
 * Construct a new Texture from file.
 * filename:  The file containing the bitmap to load.
 * tex_id:  Identifier for this texture so texture binding is possible.
 */
Texture::Texture(char * filename, int tex_id)
{
    printf("Trying to load %s\n", filename);

    // IF loading the file fails, exit
    if (!loadImage(filename)) 
    {
        exit(1);
    }  

    glBindTexture(GL_TEXTURE_2D, tex_id);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, size_x, size_y,
    0, GL_RGB, GL_UNSIGNED_BYTE, data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);   
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); 
  
}

/**
 * Load a bitmap image from file.
 * filename:  The file containing the bitmap to load.
 */
int Texture::loadImage(char * filename)
{
    FILE *file;
    unsigned long size;            // Size of the image in bytes.
    unsigned long i;               // Standard counter.
    unsigned short int planes;     // Number of planes in image (must be 1) 
    unsigned short int bpp;        // Number of bits per pixel (must be 24)
    char temp;                     // Nsed to convert bgr to rgb color.

    // IF the file cannot be found, return
    if((file = fopen(filename, "rb")) == NULL) 
    {
        printf("File Not Found : %s\n",filename);
        return 0;
    }

    // Seek through the bmp header, up to the width height
    fseek(file, 18, SEEK_CUR);

    // Read the width and height    
    size_x = getint (file);
    size_y = getint (file);

    // Calculate the size (assuming 24 bits or 3 bytes per pixel).
    size = size_x * size_y * 3;

    // Read the planes, return if not one
    planes = getshort(file);
    if (planes != 1) 
    {
        printf("Planes from %s is not 1: %u\n", filename, planes);
        return 0;
    }

    // Read the bpp, return if not 24
    bpp = getshort(file);
    if (bpp != 24) 
    {
        printf("Bpp from %s is not 24: %u\n", filename, bpp);
        return 0;
    }

    // Seek past the rest of the bitmap header
    fseek(file, 24, SEEK_CUR);

    // Read the data.
    data = (char *) malloc(size);
    if (data == NULL) 
    {
        printf("Error allocating memory for color-corrected image data");
        return 0; 
    }

    if ((i = fread(data, size, 1, file)) != 1) 
    {
        printf("Error reading image data from %s.\n", filename);
        return 0;
    }

    // Reverse all of the colors. (bgr -> rgb)
    for (i=0;i<size;i+=3) 
    { 
        temp = data[i];
        data[i] = data[i+2];
        data[i+2] = temp;
    }

    fclose(file);

    return 1;
}

/**
 * Get the next short from a file pointer.
 * fp:  File pointer to retrieve from.
 */
static unsigned int getshort(FILE *fp)
{
    int c, c1;

    // Get 2 bytes
    c = getc(fp);  
    c1 = getc(fp);

    return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/**
 * Get the next int from a file pointer.
 * fp:  File pointer to retrieve from.
 */
static unsigned int getint(FILE *fp) 
{
    int c, c1, c2, c3;

    // Get 4 bytes 
    c = getc(fp);  
    c1 = getc(fp);  
    c2 = getc(fp);  
    c3 = getc(fp);

    return ((unsigned int) c) + (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) + (((unsigned int) c3) << 24);
}