/**
 * main_release.cpp
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 *
 * Original Source: My Program 4 code.
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#include "include.h"

using namespace std;

/** Constants */
// Debugging flag (print out variable values to the console)
const int __DEBUG = 1;
const float PI = 3.14159265358979323846; // PI
const float MAX_PITCH = 50.0;     // Maximum pitch angle
const float MIN_PITCH = -50.0;    // Minimum pitch angle
const float WORLD_SIZE = 16.0;   // Length of one side of the world
const float INIT_EYE_X = 0.0;     // Initial eye x position
const float INIT_EYE_Y = 9.0;     // Initial eye y position
const float INIT_EYE_Z = 9.0;     // Initial eye z position
const float INIT_LA_X = 0.0;      // Initial look at x position
const float INIT_LA_Y = 0.0;      // Initial look at y position
const float INIT_LA_Z = 0.0;     // Initial look at z position
const float INIT_UP_X = 0.0;      // Initial up x position
const float INIT_UP_Y = 1.0;      // Initial up y position
const float INIT_UP_Z = 0.0;      // Initial up z position
const float NEAR_PLANE = 1.0;     // Near clipping plane
const float FAR_PLANE = 40.0;     // Far clipping plane
const float CAMERA_MOVE = 0.5;    // Amount of camera movement per keypress

// What a full window movement translates to in angles
const float MOUSE_ANGLE = 180.0;

/** Flags */
int grid;                         // Toggle drawing the ground plane grid
int anim;                         // Toggle model animations
int worldTex;                     // Toggle drawing the world as a texture mapping

/** Other global variables */
int GW;              // Global width
int GH;              // Global height
float alpha;         // Pitch
float beta;          // Yaw
float omega;         // Angle between the original view and the current camera position
int theta;           // Angle for animations
int mx, my;          // Mouse position in pixel coordinates
RO_Monk * monk;      // Global monk for drawing
RO_Poring * poring;  // Global poring for drawing
Texture * worldFloor;// Texture of the world floor

// Vector for camera eye, look at, and up vector
Vector3 eye, la, up;

/** Globals for lighting */
GLfloat light_pos[4] = {0.0, 15.0, 0.0, 1.0}; // Light position

// White light color
GLfloat light_amb[4] = {0.6, 0.6, 0.6, 1.0};  // Light ambience component
GLfloat light_diff[4] = {0.5, 0.5, 0.5, 1.0}; // Light diffuse component
GLfloat light_spec[4] = {0.0, 0.0, 0.0, 1.0}; // Light specular component

/**
 * Converts an angle in radians to degrees.
 * radian:  the angle to convert
 */
float radToDeg(float radian)
{
    return radian * 180 / PI;
}

/**
 * Converts an angle in degrees to radians.
 * radian:  the angle to convert
 */
float degToRad(float degree)
{
    return degree / 180 * PI;
}

/**
 * Initialization calls for opengl for static light.  Lighting will still have
 * to be enabled in order for this to work.
 */
void init_lighting()
{
    // Turn on light0
    glEnable(GL_LIGHT0);

    // Set up the diffuse, ambient and specular components for the light
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);

    // Specify our lighting model as 1 normal per face
    glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);
}

/**
 *  Set the light's position.
 */
void pos_light()
{
    glMatrixMode(GL_MODELVIEW);
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
}

/**
 * This function allows image scaling upon window reshape.
 * w:  width of the new window size
 * h:  height of the new window size
 */
void reshape(int w, int h)
{
    GW = w;
    GH = h;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(110.0, 1.0, NEAR_PLANE, FAR_PLANE);

    glMatrixMode(GL_MODELVIEW);
    glViewport(0, 0, w, h);
    glutPostRedisplay();
}

/**
 * Draw the plane of the world and a position grid.
 */
void drawWorld()
{
    // Draw the grid if toggled on
    if(grid == 1)
    {
        glPushMatrix();
        m.materials(m.black);
        for(int i = -WORLD_SIZE/2; i <= WORLD_SIZE/2; i += 2.0)
        {
            // Horizontal grid
            glBegin(GL_LINES);
            glVertex3f(-1*WORLD_SIZE/2, 0.0, i);
            glVertex3f(WORLD_SIZE/2, 0.0, i);
            glEnd();

            // Vertical grid
            glBegin(GL_LINES);
            glVertex3f(i, 0.0, -1*WORLD_SIZE/2);
            glVertex3f(i, 0.0, WORLD_SIZE/2);
            glEnd();
        }
        glPopMatrix();  
    }
    
    // IF drawing the world as a texture-mapped area
    if(worldTex)
    {
        // Draw the world plane (textured)
        glPushMatrix();
            glBindTexture(GL_TEXTURE_2D, 2);
            for(int i = -WORLD_SIZE/2; i < WORLD_SIZE/2; i += 2.0)
            {
                for(int j = -WORLD_SIZE/2; j < WORLD_SIZE/2; j += 2.0)
                {
                    glBegin(GL_POLYGON);
                    m.materials(m.white_solid);
                    glTexCoord2f(0.0, 1.0);
                    glVertex3f(i, -0.1, j);
                    glTexCoord2f(1.0, 1.0);
                    glVertex3f(i + 2.0, -0.1, j);
                    glTexCoord2f(1.0, 0.0);
                    glVertex3f(i + 2.0, -0.1, j + 2.0);
                    glTexCoord2f(0.0, 0.0);
                    glVertex3f(i, -0.1, j + 2.0);
                    glEnd();
                }
            }
            glBindTexture(GL_TEXTURE_2D, 0);
        glPopMatrix();
    }
    // ELSE draw the world plane (no texture)
    else
    {
        glPushMatrix();
        
            glBegin(GL_POLYGON);
            m.materials(m.jade);
            glNormal3f(0.0, 1.0, 0.0);
            glVertex3f(-1*WORLD_SIZE/2, -0.1, WORLD_SIZE/2);
            glVertex3f(WORLD_SIZE/2, -0.1, WORLD_SIZE/2);
            glVertex3f(WORLD_SIZE/2, -0.1, -1*WORLD_SIZE/2);
            glVertex3f(-1*WORLD_SIZE/2, -0.1, -1*WORLD_SIZE/2);
            glEnd();
        glPopMatrix();
    }

    // Draw a sphere where the light is
    glPushMatrix();
        m.materials(m.gold);
        glTranslatef(light_pos[0], light_pos[1], light_pos[2]);
        glutSolidSphere(1.0, 100, 100);
    glPopMatrix();
}

/**
 * The idle callback function.
 */
void idle()
{
    // IF animations are toggled on
    if(anim == 1)
    {
        theta = (theta + 1)%360;
        glutPostRedisplay();
    }
}

/**
 * The display call back.  All drawing should be done in this function.
 */
void display()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    
        // Set up the camera
        gluLookAt(eye.x, eye.y, eye.z,  // Eye position
                  la.x, la.y, la.z,  // Point we are looking at
                  up.x, up.y, up.z); // Up vector

        // Position the light in the scene
        pos_light();

        // Draw the world and objects
        drawWorld();

        // Get the angle between the camera position and the original position
        omega = radToDeg(getAngle(Vector3(eye.x/sqrt(eye.x*eye.x+eye.z*eye.z), 
            0, eye.z/sqrt(eye.x*eye.x+eye.z*eye.z)), Vector3(0, 0, 1)));
        // Negate backward angles 
        if(eye.x < 0)
        {
            omega *= -1;
        }
        
        // Draw the monk and poring
        monk->draw(theta, omega);
        poring->draw(theta, omega);
    glPopMatrix();
    glutSwapBuffers();
}

/**
 * Print mouse and keyboard functions to the console.
 */
void printHelp()
{
    printf("\n*****Final Project Help*****\n");

    printf("\nKeyboard commands:\n");

    printf("\tz: Summon Spirit Sphere\n");
    printf("\tx: Fury\n");
    printf("\tc: Guillotine Fist\n");
    printf("\t=: Increase monk DEX\n");
    printf("\t-: Decrease monk DEX\n");
    printf("\ta/A: Rotate left of the current gaze\n");
    printf("\td/D: Rotate right of the current gaze\n");
    printf("\tw/W: Move forward in the scene along the current gaze\n");
    printf("\ts/S: Move backward in the scene along the current gaze\n");
    printf("\tg/G: Toggle drawing the world grid\n");
    printf("\tm/M: Toggle world texture mapping\n");
    printf("\ti: Toggle a few animations\n");
    printf("\tr/R: Reset the camera\n");
    printf("\th/H: Print this help menu\n");
    printf("\tq: Quit\n");
}

/**
 * Updates the camera when a movement key is pressed.
 * x:  Movement direction on the x-axis
 * y:  Movement direction on the y-axis
 * z:  Movement direction on the z-axis
 */
void updateCamera(float x, float y, float z)
{
    // Gaze vector, three camera axes, intermediate calculation vector
    Vector3 gaze, u, v, w, temp;
    float gazeLen, uLen;    // Length of gaze and u vectors

    gaze = Vector3(la.x - eye.x, la.y - eye.y, la.z - eye.z);
    gazeLen = getVectorLength(gaze);
    // Calculate u, v, w
    w = Vector3(gaze.x/gazeLen, gaze.y/gazeLen, gaze.z/gazeLen);
    
    crossProd(w, up, temp);
    uLen = getVectorLength(temp);
    u = Vector3(temp.x/uLen, temp.y/uLen, temp.z/uLen);

    crossProd(w, u, v);

    // Update eye and la
    eye.x += CAMERA_MOVE*(u.x * x + v.x * y + w.x * z);
    eye.y += CAMERA_MOVE*(u.y * x + v.y * y + w.y * z);
    eye.z += CAMERA_MOVE*(u.z * x + v.z * y + w.z * z);
}

/**
 * This function provides the mouse callback.
 * button:  the mouse button that triggered the callback (l or r)
 * state:  the state of the button (up or down)
 * x:  the x position of the mouse
 * y:  the y position of the mouse
 */
void mouse(int button, int state, int x, int y)
{
    if (button == GLUT_LEFT_BUTTON) 
    {
        // IF the left button is clicked
        if (state == GLUT_DOWN) 
        {
            if(__DEBUG == 1)
            {
                printf("mouse clicked at %d %d\n", x, y);
            }
            mx = x;
            my = y;
        }
    }
}

/**
 * This function provides the mouse movement callback.
 * int x:  the x position of the mouse
 * int y:  the y position of the mouse
 */
void mouseMove(int x, int y)
{
    if(__DEBUG == 1)
    {
        printf("mouse moved at %d %d\n", x, y);
    }
}

/**
 * This function provides the keyboard callback.
 * key:  the key that was pressed
 * x:  ?
 * y:  ?
 */
void keyboard(unsigned char key, int x, int y ){
    switch( key )
    {
        // Summon a spirit sphere for the monk
        case 'z':
            monk->summonSpiritSphere();
            break;
        // Make the monk cast Fury
        case 'x':
            monk->fury();
            break;
        // Make the monk cast Guillotine Fist
        case 'c':
            monk->guillotineFist();
            break;
        // Increase the monk's DEX stat
        case '=':
            monk->incDex();
            break;
        // Decrease the monk's DEX stat
        case '-':
            monk->decDex();
            break;
        // Rotate left of the current gaze
        case 'a': case 'A' :
            updateCamera(-1.0, 0.0, 0.0);
            break;
        // Rotate right of the current gaze
        case 'd': case 'D' :
            updateCamera(1.0, 0.0, 0.0);
            break;
        // Move forward in the scene along the current gaze
        case 'w': case 'W' :
            updateCamera(0.0, 0.0, 1.0);
            break;
        // Move backward in the scene along the current gaze
        case 's': case 'S' :
            updateCamera(0.0, 0.0, -1.0);
            break;
        // Toggle the world grid
        case 'g': case 'G' :
            grid = !grid;
            break;
        // Toggle texture mapping the world floor
        case 'm': case 'M' :
            worldTex = !worldTex;
            break;
        case 'i':
            anim = !anim;
            break;
        // Reset the camera back to its original position and orientation
        case 'r': case 'R' :
            eye = Vector3(INIT_EYE_X, INIT_EYE_Y, INIT_EYE_Z);
            la = Vector3(INIT_LA_X, INIT_LA_Y, INIT_LA_Z);
            alpha = 90;
            beta = 45;
            break;
        // Help
        case 'h': case 'H' :
            printHelp();
            break;
        // Quit
        case 'q': case 'Q' :
            delete monk;
            delete poring;
            delete worldFloor;
            exit( EXIT_SUCCESS );
            break;
    }
    glutPostRedisplay();
}

int main( int argc, char** argv )
{
    // Set up my window
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(900, 900);
    glutInitWindowPosition(100, 100);
    glutCreateWindow("Final Project:  Ragnarok Online Monk Rendering");
    glClearColor(1.0, 1.0, 1.0, 1.0);

    // Register glut callback functions and menus
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glutKeyboardFunc(keyboard);
    glutMouseFunc(mouse);
    glutMotionFunc(mouseMove);
    glutIdleFunc(idle);

    // Enable z-buffe
    glEnable(GL_DEPTH_TEST);
    // Enable texture mapping
    glEnable(GL_TEXTURE_2D);
    
    // Initialization
    eye = Vector3(INIT_EYE_X, INIT_EYE_Y, INIT_EYE_Z);
    la = Vector3(INIT_LA_X, INIT_LA_Y, INIT_LA_Z);
    up = Vector3(INIT_UP_X, INIT_UP_Y, INIT_UP_Z);
    alpha = 90;
    beta = 45;
    theta = 0;
    grid = anim = worldTex = 1;
    m.init_materials();
    monk = new RO_Monk();
    poring = new RO_Poring(monk);
    worldFloor = new Texture("floor.bmp", 2);

    init_lighting();
    glutMainLoop();
}
